RecvError

Enum RecvError 

pub enum RecvError {
    MaxMessageSizeExceeded {
        meta: Meta,
    },
    Io {
        source: Error,
        meta: Meta,
    },
}
Error when receiving a oneshot or mpsc message. For local communication, the only thing that can go wrong is that the sender has been closed.

For rpc communication, there can be any number of errors, so this is a generic io error.

Variants



MaxMessageSizeExceeded

The message exceeded the maximum allowed message size (see MAX_MESSAGE_SIZE).

Fields

meta: Meta

Captured call-site metadata



Io

An io error occurred. This can occur for remote communication, due to a network error or deserialization error.

Fields

source: Error
meta: Meta

Captured call-site metadata