Sender

Enum Sender 

pub enum Sender<T> {
    Tokio(Sender<T>),
    Boxed(Arc<dyn DynSender<T>>),
}
Single producer, single consumer sender.

For the local case, this wraps a tokio::sync::mpsc::Sender.

Variants



Tokio(Sender<T>)



Boxed(Arc<dyn DynSender<T>>)

Implementations

Source

impl<T> Sender<T>

pub fn is_rpc(&self) -> bool
where T: 'static,

pub fn into_sink(self) -> impl Sink<T, Error = SendError> + Send + 'static
where T: Send + Sync + 'static,

Available on crate feature stream only.
Source

impl<T: Send + Sync + 'static> Sender<T>

pub fn with_filter<F>(self, f: F) -> Sender<T>
where F: Fn(&T) -> bool + Send + Sync + 'static,

Applies a filter before sending.

Messages that don’t pass the filter are dropped.

If you want to combine multiple filters and maps with minimal overhead, use with_filter_map directly.

pub fn with_map<U, F>(self, f: F) -> Sender<U>
where F: Fn(U) -> T + Send + Sync + 'static, U: Send + Sync + 'static,

Applies a transform before sending.

If you want to combine multiple filters and maps with minimal overhead, use with_filter_map directly.

pub fn with_filter_map<U, F>(self, f: F) -> Sender<U>
where F: Fn(U) -> Option<T> + Send + Sync + 'static, U: Send + Sync + 'static,

Applies a filter and transform before sending.

Any combination of filters and maps can be expressed using a single filter_map.

pub async fn closed(&self)

Future that resolves when the sender is closed

Source

impl<T: Send + 'static> Sender<T>

pub async fn send(&self, value: T) -> Result<(), SendError>

Send a message and yield until either it is sent or an error occurs.

Cancellation safety

If the future is dropped before completion, and if this is a remote sender, then the sender will be closed and further sends will return an SendError::Io with std::io::ErrorKind::BrokenPipe. Therefore, make sure to always poll the future until completion if you want to reuse the sender or any clone afterwards.

pub async fn try_send(&self, value: T) -> Result<bool, SendError>

Try to send a message, returning as fast as possible if sending is not currently possible. This can be used to send ephemeral messages.

For the local case, this will immediately return false if the channel is full.

For the remote case, it will attempt to send the message and return false if sending the first byte fails, otherwise yield until the message is completely sent or an error occurs. This guarantees that the message is sent either completely or not at all.

Returns true if the message was sent.

Cancellation safety

If the future is dropped before completion, and if this is a remote sender, then the sender will be closed and further sends will return an SendError::Io with std::io::ErrorKind::BrokenPipe. Therefore, make sure to always poll the future until completion if you want to reuse the sender or any clone afterwards.