WithChannels

Struct WithChannels 

pub struct WithChannels<I: Channels<S>, S: Service> {
    pub inner: I,
    pub tx: <I as Channels<S>>::Tx,
    pub rx: <I as Channels<S>>::Rx,
    pub span: Span,
}
A wrapper for a message with channels to send and receive it. This expands the protocol message to a full message that includes the active and unserializable channels.

The channel kind for rx and tx is defined by implementing the Channels trait, either manually or using a macro.

When the spans feature is enabled, this also includes a tracing span to carry the tracing context during message passing.

Fields

inner: I

The inner message.

tx: <I as Channels<S>>::Tx

The return channel to send the response to. Can be set to crate::channel::none::NoSender if not needed.

rx: <I as Channels<S>>::Rx

The request channel to receive the request from. Can be set to NoReceiver if not needed.

span: Span
Available on crate feature spans only.

The current span where the full message was created.

Implementations

Source

impl<I: Channels<S>, S: Service> WithChannels<I, S>

pub fn parent_span_opt(&self) -> Option<&Span>

Available on crate feature spans only.

Get the parent span